@@ -30,8 +30,8 @@
 #include <KItinerary/DocumentUtil>
 #include <KItinerary/Event>
 #include <KItinerary/ExtractorCapabilities>
-#include <KItinerary/ExtractorEngine>
 #include <KItinerary/ExtractorDocumentNode>
+#include <KItinerary/ExtractorEngine>
 #include <KItinerary/ExtractorPostprocessor>
 #include <KItinerary/File>
 #include <KItinerary/JsonLdDocument>
@@ -55,15 +55,15 @@
 #include <QMimeData>
 #include <QMimeDatabase>
 #include <QNetworkAccessManager>
-#include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QNetworkRequest>
 #include <QScopedValueRollback>
 #include <QStandardPaths>
 #include <QTemporaryDir>
 #include <QTemporaryFile>
-#include <QUuid>
 #include <QUrl>
 #include <QUrlQuery>
+#include <QUuid>
 
 #ifdef Q_OS_ANDROID
 #include "android/itineraryactivity.h"
@@ -80,9 +80,9 @@
 using namespace Qt::Literals::StringLiterals;
 using namespace KItinerary;
 
-ApplicationController* ApplicationController::s_instance = nullptr;
+ApplicationController *ApplicationController::s_instance = nullptr;
 
-ApplicationController::ApplicationController(QObject* parent)
+ApplicationController::ApplicationController(QObject *parent)
     : QObject(parent)
 {
     s_instance = this;
@@ -93,38 +93,38 @@ ApplicationController::~ApplicationController()
     s_instance = nullptr;
 }
 
-void ApplicationController::setNetworkAccessManagerFactory(const std::function<QNetworkAccessManager*()> &namFactory)
+void ApplicationController::setNetworkAccessManagerFactory(const std::function<QNetworkAccessManager *()> &namFactory)
 {
     m_namFactory = namFactory;
 }
 
-ApplicationController* ApplicationController::instance()
+ApplicationController *ApplicationController::instance()
 {
     return s_instance;
 }
 
-void ApplicationController::requestOpenPage(const QString &page)
+void ApplicationController::requestOpenPage(const QString &page, const QVariantMap &properties)
 {
-    Q_EMIT openPageRequested(page);
+    Q_EMIT openPageRequested(page, properties);
 }
 
-void ApplicationController::setReservationManager(ReservationManager* resMgr)
+void ApplicationController::setReservationManager(ReservationManager *resMgr)
 {
     m_resMgr = resMgr;
 }
 
-void ApplicationController::setPkPassManager(PkPassManager* pkPassMgr)
+void ApplicationController::setPkPassManager(PkPassManager *pkPassMgr)
 {
     m_pkPassMgr = pkPassMgr;
     connect(m_pkPassMgr, &PkPassManager::passUpdated, this, &ApplicationController::pkPassUpdated);
 }
 
-void ApplicationController::setDocumentManager(DocumentManager* docMgr)
+void ApplicationController::setDocumentManager(DocumentManager *docMgr)
 {
     m_docMgr = docMgr;
 }
 
-void ApplicationController::setTransferManager(TransferManager* transferMgr)
+void ApplicationController::setTransferManager(TransferManager *transferMgr)
 {
     m_transferMgr = transferMgr;
 }
@@ -161,6 +161,8 @@ void ApplicationController::handleIntent(const KAndroidExtras::Intent &intent)
         if (url.scheme() == "page"_L1) {
             qCDebug(Log) << url;
             requestOpenPage(url.path().mid(1));
+        } else if (url.scheme() == "geo"_L1) {
+            handleGeoUrl(url);
         }
     }
 #else
@@ -168,6 +170,25 @@ void ApplicationController::handleIntent(const KAndroidExtras::Intent &intent)
 #endif
 }
 
+void ApplicationController::handleGeoUrl(const QUrl &url)
+{
+    if (url.scheme() != "geo"_L1) {
+        qCWarning(Log) << "Not a geo url!" << url;
+    }
+    const auto pathElems = url.path().split(QLatin1Char(';'));
+    const auto coordElems = pathElems.isEmpty() ? QStringList() : pathElems.at(0).split(QLatin1Char(','));
+    const auto lat = coordElems.size() < 2 ? 0.0 : coordElems.at(0).toDouble();
+    const auto lon = coordElems.size() < 2 ? 0.0 : coordElems.at(1).toDouble();
+
+    const QUrlQuery geoQuery(url.query());
+    const auto query = geoQuery.queryItemValue(u"q"_s);
+
+    KPublicTransport::Location location;
+    location.setCoordinate(lat, lon);
+    location.setName(query.isEmpty() ? i18nc("geo coordinate", "%1, %2", lat, lon) : query);
+    requestOpenPage(u"journeyRequest"_s, {{u"arrivalStop"_s, location}});
+}
+
 void ApplicationController::commitImport(ImportController *importController)
 {
     qCDebug(Log);
@@ -176,6 +197,8 @@ void ApplicationController::commitImport(ImportController *importController)
     int passCount = 0;
     int healthCertCount = 0;
 
+    TripGroupingBlocker groupBlocker(m_tripGroupMgr);
+    QStringList tripGroupElements;
     for (const auto &elem : importController->elements()) {
         if (!elem.selected) {
             continue;
@@ -183,60 +206,63 @@ void ApplicationController::commitImport(ImportController *importController)
 
         QVariantList docIds;
         switch (elem.type) {
-            case ImportElement::Reservation:
-                m_resMgr->addReservation(elem.updateData.isNull() ? elem.data : elem.updateData, elem.id);
-                docIds = DocumentUtil::documentIds(elem.data);
-                for (const auto &r : elem.batch) {
-                    m_resMgr->addReservation(r);
-                    docIds += DocumentUtil::documentIds(r);
+        case ImportElement::Reservation:
+            tripGroupElements.push_back(m_resMgr->batchForReservation(m_resMgr->addReservation(elem.updateData.isNull() ? elem.data : elem.updateData, elem.id)));
+            docIds = DocumentUtil::documentIds(elem.data);
+            for (const auto &r : elem.batch) {
+                m_resMgr->addReservation(r);
+                docIds += DocumentUtil::documentIds(r);
+            }
+
+            if (elem.bundleIdx >= 0 && !elem.id.isEmpty()) {
+                const auto bundle = importController->bundles()[elem.bundleIdx].data.get();
+
+                auto t = Transfer::fromJson(
+                    QJsonDocument::fromJson(bundle->customData(BUNDLE_TRANSFER_DOMAIN, Transfer::identifier(elem.id, Transfer::Before))).object());
+                m_transferMgr->importTransfer(t);
+                t = Transfer::fromJson(
+                    QJsonDocument::fromJson(bundle->customData(BUNDLE_TRANSFER_DOMAIN, Transfer::identifier(elem.id, Transfer::After))).object());
+                m_transferMgr->importTransfer(t);
+
+                const auto obj = QJsonDocument::fromJson(bundle->customData(BUNDLE_LIVE_DATA_DOMAIN, elem.id)).object();
+                if (!obj.isEmpty()) {
+                    auto ld = LiveData::fromJson(obj);
+                    ld.store(elem.id);
+                    m_liveDataMgr->importData(elem.id, std::move(ld));
                 }
+            }
 
-                if (elem.bundleIdx >= 0 && !elem.id.isEmpty()) {
-                    const auto bundle = importController->bundles()[elem.bundleIdx].data.get();
-
-                    auto t = Transfer::fromJson(QJsonDocument::fromJson(bundle->customData(BUNDLE_TRANSFER_DOMAIN, Transfer::identifier(elem.id, Transfer::Before))).object());
-                    m_transferMgr->importTransfer(t);
-                    t = Transfer::fromJson(QJsonDocument::fromJson(bundle->customData(BUNDLE_TRANSFER_DOMAIN, Transfer::identifier(elem.id, Transfer::After))).object());
-                    m_transferMgr->importTransfer(t);
-
-                    const auto obj = QJsonDocument::fromJson(bundle->customData(BUNDLE_LIVE_DATA_DOMAIN, elem.id)).object();
-                    if (!obj.isEmpty()) {
-                        auto ld = LiveData::fromJson(obj);
-                        ld.store(elem.id);
-                        m_liveDataMgr->importData(elem.id, std::move(ld));
-                    }
-                }
-
-                ++reservationCount;
-                break;
-            case ImportElement::Pass:
-                docIds = DocumentUtil::documentIds(elem.data);
-                m_passMgr->import(elem.data, elem.id);
-                ++passCount;
-                break;
-            case ImportElement::HealthCertificate:
-                m_healthCertMgr->importCertificate(HealthCertificateManager::certificateRawData(elem.data));
-                ++healthCertCount;
-                break;
-            case ImportElement::Template:
-                if (JsonLd::isA<LodgingBusiness>(elem.data)) { // TODO can't we do the reservation promotion in ImportController and share the model representation with reservations?
-                    LodgingReservation res;
-                    res.setReservationFor(elem.data);
-                    res.setPotentialAction(elem.data.value<LodgingBusiness>().potentialAction());
-                    Q_EMIT editNewHotelReservation(res);
-                } else if (JsonLd::isA<FoodEstablishment>(elem.data) || JsonLd::isA<LocalBusiness>(elem.data)) {
-                    // LocalBusiness is frequently used for restaurants in website annotations
-                    FoodEstablishmentReservation res;
-                    res.setReservationFor(elem.data);
-                    res.setPotentialAction(JsonLd::convert<Organization>(elem.data).potentialAction());
-                    Q_EMIT editNewRestaurantReservation(res);
-                } else if (JsonLd::isA<EventReservation>(elem.data)) {
-                    Q_EMIT editNewEventReservation(elem.data);
-                }
-                break;
-            case ImportElement::Backup:
-                importBundle(importController->bundles()[elem.bundleIdx].data.get());
-                break;
+            ++reservationCount;
+            break;
+        case ImportElement::Pass:
+            docIds = DocumentUtil::documentIds(elem.data);
+            m_passMgr->import(elem.data, elem.id);
+            ++passCount;
+            break;
+        case ImportElement::HealthCertificate:
+            m_healthCertMgr->importCertificate(HealthCertificateManager::certificateRawData(elem.data));
+            ++healthCertCount;
+            break;
+        case ImportElement::Template:
+            if (JsonLd::isA<LodgingBusiness>(
+                    elem.data)) { // TODO can't we do the reservation promotion in ImportController and share the model representation with reservations?
+                LodgingReservation res;
+                res.setReservationFor(elem.data);
+                res.setPotentialAction(elem.data.value<LodgingBusiness>().potentialAction());
+                Q_EMIT editNewHotelReservation(res);
+            } else if (JsonLd::isA<FoodEstablishment>(elem.data) || JsonLd::isA<LocalBusiness>(elem.data)) {
+                // LocalBusiness is frequently used for restaurants in website annotations
+                FoodEstablishmentReservation res;
+                res.setReservationFor(elem.data);
+                res.setPotentialAction(JsonLd::convert<Organization>(elem.data).potentialAction());
+                Q_EMIT editNewRestaurantReservation(res);
+            } else if (JsonLd::isA<EventReservation>(elem.data)) {
+                Q_EMIT editNewEventReservation(elem.data);
+            }
+            break;
+        case ImportElement::Backup:
+            importBundle(importController->bundles()[elem.bundleIdx].data.get());
+            break;
         }
 
         for (const auto &docId : docIds) {
@@ -257,6 +283,26 @@ void ApplicationController::commitImport(ImportController *importController)
         }
     }
 
+    // add or update trip group
+    QString tripGroupId;
+    if (!tripGroupElements.isEmpty() && m_tripGroupMgr) { // TODO many unit tests don't provide m_tripGroupMgr, fix this eventually
+        if (!importController->tripGroupId().isEmpty()) {
+            tripGroupId = importController->tripGroupId();
+            m_tripGroupMgr->addToGroup(tripGroupElements, tripGroupId);
+        } else if (!importController->tripGroupName().isEmpty()) {
+            tripGroupId = m_tripGroupMgr->createGroup(tripGroupElements, importController->tripGroupName());
+        }
+
+        if (!tripGroupId.isEmpty() && !importController->tripGroup().matrixRoomId().isEmpty()) {
+            auto tg = m_tripGroupMgr->tripGroup(tripGroupId);
+            if (tg.matrixRoomId().isEmpty()) {
+                tg.setMatrixRoomId(importController->tripGroup().matrixRoomId());
+                m_tripGroupMgr->updateTripGroup(tripGroupId, tg);
+            }
+        }
+    }
+    // TODO open the created/modified trip group afterwards?
+
     importController->clearSelected();
 
     if (reservationCount) {
@@ -264,13 +310,20 @@ void ApplicationController::commitImport(ImportController *importController)
     }
     if (passCount) {
         Q_EMIT infoMessage(i18np("One pass imported.", "%1 passes imported.", passCount));
-
     }
     if (healthCertCount) {
         Q_EMIT infoMessage(i18np("One health certificate imported.", "%1 health certificates imported.", healthCertCount));
     }
 }
 
+QString ApplicationController::addNewReservation(const QVariant &res, const QString &tgId)
+{
+    TripGroupingBlocker blocker(m_tripGroupMgr);
+    const auto resId = m_resMgr->addReservationWithPostProcessing(res);
+    m_tripGroupMgr->addToGroup({resId}, tgId);
+    return resId;
+}
+
 void ApplicationController::exportToFile(const QUrl &url)
 {
     if (url.isEmpty()) {
@@ -289,7 +342,7 @@ void ApplicationController::exportToFile(const QUrl &url)
     exporter.exportReservations(m_resMgr);
     exporter.exportPasses(m_pkPassMgr);
     exporter.exportDocuments(m_docMgr);
-    exporter.exportFavoriteLocations(m_favLocModel);
+    exporter.exportFavoriteLocations(m_favLocModel->favoriteLocations());
     exporter.exportTransfers(m_resMgr, m_transferMgr);
     exporter.exportTripGroups(m_tripGroupMgr);
     exporter.exportPasses(m_passMgr);
@@ -348,11 +401,12 @@ bool ApplicationController::exportTripToFile(const QString &tripGroupId, const Q
     const auto batchIds = tg.elements();
 
     QSet<QString> docIdSet;
+    std::vector<FavoriteLocation> favoriteLocations;
 
     Exporter exporter(&f);
     for (const auto &batchId : batchIds) {
         exporter.exportReservationBatch(m_resMgr, batchId);
-        exporter.exportTransfersForBatch(batchId, m_transferMgr);
+        exporter.exportTransfersForBatch(batchId, m_transferMgr, favoriteLocations);
         exporter.exportLiveDataForBatch(batchId);
 
         const auto resIds = m_resMgr->reservationsForBatch(batchId);
@@ -373,6 +427,7 @@ bool ApplicationController::exportTripToFile(const QString &tripGroupId, const Q
         }
     }
     exporter.exportTripGroup(tripGroupId, tg);
+    exporter.exportFavoriteLocations(favoriteLocations);
     return true;
 }
 
@@ -447,10 +502,11 @@ bool ApplicationController::exportBatchToFile(const QString &batchId, const QStr
     }
 
     QSet<QString> docIdSet;
+    std::vector<FavoriteLocation> favoriteLocations;
 
     Exporter exporter(&f);
     exporter.exportReservationBatch(m_resMgr, batchId);
-    exporter.exportTransfersForBatch(batchId, m_transferMgr);
+    exporter.exportTransfersForBatch(batchId, m_transferMgr, favoriteLocations);
     exporter.exportLiveDataForBatch(batchId);
 
     const auto resIds = m_resMgr->reservationsForBatch(batchId);
@@ -469,6 +525,7 @@ bool ApplicationController::exportBatchToFile(const QString &batchId, const QStr
             }
         }
     }
+    exporter.exportFavoriteLocations(favoriteLocations);
     return true;
 }
 
@@ -477,12 +534,14 @@ bool ApplicationController::importBundle(KItinerary::File *file)
     Importer importer(file);
     int count = 0;
     {
+        TripGroupingBlocker groupingBlocker(m_tripGroupMgr);
         QSignalBlocker blocker(this); // suppress infoMessage()
         count += importer.importReservations(m_resMgr);
         count += importer.importPasses(m_pkPassMgr);
         count += importer.importDocuments(m_docMgr);
         count += importer.importFavoriteLocations(m_favLocModel);
         count += importer.importTransfers(m_resMgr, m_transferMgr);
+        count += importer.importTripGroups(m_tripGroupMgr);
         count += importer.importPasses(m_passMgr);
         count += importer.importHealthCertificates(healthCertificateManager());
         count += importer.importLiveData(m_liveDataMgr);
@@ -504,7 +563,7 @@ void ApplicationController::pkPassUpdated(const QString &passId)
 
     const auto pass = m_pkPassMgr->pass(passId);
     KItinerary::ExtractorEngine engine;
-    engine.setContent(QVariant::fromValue<KPkPass::Pass*>(pass), u"application/vnd.apple.pkpass");
+    engine.setContent(QVariant::fromValue<KPkPass::Pass *>(pass), u"application/vnd.apple.pkpass");
     m_resMgr->addReservationsWithPostProcessing(JsonLdDocument::fromJson(engine.extract()));
 }
 
@@ -618,12 +677,22 @@ bool ApplicationController::hasHealthCertificateSupport() const
     return HealthCertificateManager::isAvailable();
 }
 
-HealthCertificateManager* ApplicationController::healthCertificateManager() const
+HealthCertificateManager *ApplicationController::healthCertificateManager() const
 {
     if (!m_healthCertMgr) {
-        m_healthCertMgr = new HealthCertificateManager(const_cast<ApplicationController*>(this));
+        m_healthCertMgr = new HealthCertificateManager(const_cast<ApplicationController *>(this));
     }
     return m_healthCertMgr;
 }
 
+void ApplicationController::setContextTripGroupId(const QString &contextTripGroupId)
+{
+    if (m_contextTripGroupId == contextTripGroupId) {
+        return;
+    }
+
+    m_contextTripGroupId = contextTripGroupId;
+    Q_EMIT contextTripGroupIdChanged();
+}
+
 #include "moc_applicationcontroller.cpp"
