@@ -5,11 +5,11 @@
 */
 
 #include "tripgroupmanager.h"
-#include "tripgroup.h"
 #include "constants.h"
 #include "logging.h"
 #include "reservationmanager.h"
 #include "transfermanager.h"
+#include "tripgroup.h"
 
 #include <KItinerary/Event>
 #include <KItinerary/LocationUtil>
@@ -31,18 +31,17 @@ using namespace Qt::Literals::StringLiterals;
 using namespace KItinerary;
 
 constexpr inline const auto MaximumTripDuration = 20; // in days
-constexpr inline const auto MaximumTripElements = 20;
-constexpr inline const auto MinimumTripElements = 2;
+constexpr inline const auto MaximumTripElements = 30;
 
-TripGroupManager::TripGroupManager(QObject* parent) :
-    QObject(parent)
+TripGroupManager::TripGroupManager(QObject *parent)
+    : QObject(parent)
 {
     load();
 }
 
 TripGroupManager::~TripGroupManager() = default;
 
-ReservationManager* TripGroupManager::reservationManager() const
+ReservationManager *TripGroupManager::reservationManager() const
 {
     return m_resMgr;
 }
@@ -73,7 +72,9 @@ void TripGroupManager::setTransferManager(TransferManager *transferMgr)
 {
     m_transferMgr = transferMgr;
 
-    const auto transferChangedWrapper = [this](const Transfer &t) { transferChanged(t.reservationId(), t.alignment()); };
+    const auto transferChangedWrapper = [this](const Transfer &t) {
+        transferChanged(t.reservationId(), t.alignment());
+    };
     connect(m_transferMgr, &TransferManager::transferAdded, this, transferChangedWrapper);
     connect(m_transferMgr, &TransferManager::transferChanged, this, transferChangedWrapper);
     connect(m_transferMgr, &TransferManager::transferRemoved, this, &TripGroupManager::transferChanged);
@@ -164,11 +165,14 @@ void TripGroupManager::removeTripGroup(const QString &groupId)
         return;
     }
 
+    Q_EMIT tripGroupAboutToBeRemoved(groupId);
     for (const auto &elem : groupIt.value().elements()) {
         const auto it = m_reservationToGroupMap.find(elem);
         // check if this still points to the removed group (might not be the case if an overlapping group was added meanwhile)
         if (it != m_reservationToGroupMap.end() && it.value() == groupId) {
             m_reservationToGroupMap.erase(it);
+        } else {
+            qCWarning(Log) << "missing reservation to trip group reference" << elem;
         }
     }
     m_tripGroups.erase(groupIt);
@@ -205,18 +209,24 @@ void TripGroupManager::removeReservationsInGroup(const QString &groupId)
         return;
     }
 
-    // as we remove entries one by one we'd get and propagate update notifications for all of them
-    // so block those and emit one notifications manually once we are done
-    TripGroupingBlocker groupingBlocker(this);
-    QSignalBlocker signalBlocker(this);
+    if (!groupIt.value().elements().isEmpty()) {
+        Q_EMIT tripGroupAboutToBeRemoved(groupId);
+        // as we remove entries one by one we'd get and propagate update notifications for all of them
+        // so block those and emit one notifications manually once we are done
+        TripGroupingBlocker groupingBlocker(this);
+        QSignalBlocker signalBlocker(this);
 
-    const auto elements = groupIt.value().elements();
-    for (const auto &element : elements) {
-        m_resMgr->removeBatch(element);
+        const auto elements = groupIt.value().elements();
+        for (const auto &element : elements) {
+            m_resMgr->removeBatch(element);
+        }
+
+        signalBlocker.unblock();
+        Q_EMIT tripGroupRemoved(groupId);
+    } else {
+        // we can get here for elements.isEmpty() as well, in which case all of the above has no effect
+        removeTripGroup(groupId);
     }
-
-    signalBlocker.unblock();
-    Q_EMIT tripGroupRemoved(groupId);
 }
 
 void TripGroupManager::batchAdded(const QString &resId)
@@ -259,7 +269,7 @@ void TripGroupManager::batchRenamed(const QString &oldBatchId, const QString &ne
 
     const auto tgId = mapIt.value(); // NOLINT performance-unnecessary-copy-initialization
 
-    auto tg = tripGroup(tgId);
+    auto &tg = m_tripGroups[tgId];
     auto elems = tg.elements();
     const auto elemIt = std::find(elems.begin(), elems.end(), oldBatchId);
     (*elemIt) = newBatchId;
@@ -278,13 +288,16 @@ void TripGroupManager::batchRemoved(const QString &resId)
     const auto mapIt = m_reservationToGroupMap.constFind(resId);
     if (mapIt != m_reservationToGroupMap.constEnd()) {
         const auto groupIt = m_tripGroups.find(mapIt.value());
-        Q_ASSERT(groupIt != m_tripGroups.end());
+        if (groupIt == m_tripGroups.end()) {
+            qCWarning(Log) << "dangling trip group reference for reservation" << resId;
+            return;
+        }
         const auto groupId = groupIt.key(); // copy as the iterator might become invalid below - NOLINT performance-unnecessary-copy-initialization
 
         auto elems = groupIt.value().elements();
         elems.removeAll(resId);
-        if (elems.isEmpty() || (groupIt.value().isAutomaticallyGrouped() && elems.size() < MinimumTripElements)) { // group deleted
-            qDebug() << "removing trip group due to getting too small";
+        if (elems.isEmpty()) { // group deleted
+            qDebug() << "removing empty trip group";
             removeTripGroup(groupId);
         } else { // group changed
             qDebug() << "removing element from trip group" << resId << elems;
@@ -313,9 +326,9 @@ void TripGroupManager::transferChanged(const QString &resId, Transfer::Alignment
     const auto tgIt = m_tripGroups.find(tgId);
     assert(tgIt != m_tripGroups.end() && !tgIt.value().elements().empty());
 
-    // if tranfers leading or trailing the trip group changed, the start/end time of the group change
+    // if transfers leading or trailing the trip group changed, the start/end time of the group change
     if ((alignment == Transfer::Before && tgIt.value().elements().constFirst() == resId)
-     || (alignment == Transfer::After && tgIt.value().elements().constLast() == resId)) {
+        || (alignment == Transfer::After && tgIt.value().elements().constLast() == resId)) {
         if (recomputeTripGroupTimes(tgIt.value())) {
             tgIt.value().store(fileForGroup(tgIt.key()));
             Q_EMIT tripGroupChanged(tgId);
@@ -345,11 +358,6 @@ void TripGroupManager::scanAll()
             prevGroup = groupIt.value();
         }
 
-        // not a location change? -> continue
-        if (!LocationUtil::isLocationChange(m_resMgr->reservation(*it))) {
-            continue;
-        }
-
         scanOne(it);
         prevGroup = m_reservationToGroupMap.value(*it);
     }
@@ -358,7 +366,7 @@ void TripGroupManager::scanAll()
 static bool isConnectedTransition(const QVariant &fromRes, const QVariant &toRes)
 {
     const auto from = LocationUtil::arrivalLocation(fromRes);
-    const auto to = LocationUtil::departureLocation(toRes);
+    const auto to = LocationUtil::isLocationChange(toRes) ? LocationUtil::departureLocation(toRes) : LocationUtil::location(toRes);
     if (LocationUtil::isSameLocation(from, to, LocationUtil::CityLevel)) {
         return true;
     }
@@ -403,12 +411,12 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
 
         const auto prevRes = res;
         const auto curRes = m_resMgr->reservation(*it);
+        const auto isLocationChange = LocationUtil::isLocationChange(curRes);
 
         // not a location change? -> continue searching
-        if (!LocationUtil::isLocationChange(curRes)) {
-            continue;
+        if (isLocationChange) {
+            res = curRes;
         }
-        res = curRes;
 
         // all search strategies think they are done
         if (resNumSearchDone && connectedSearchDone && explicitSearchDone) {
@@ -423,7 +431,7 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         }
 
         // maximum trip duration exceeded?
-        const auto endDt = SortUtil::endDateTime(res);
+        const auto endDt = SortUtil::endDateTime(curRes);
         if (explicitSearchDone && beginDt.daysTo(endDt) > MaximumTripDuration) {
             qDebug() << "  aborting search, maximum trip duration reached";
             break;
@@ -431,40 +439,45 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
 
         // check for connected transitions (ie. previous arrival == current departure)
         const auto prevArrival = LocationUtil::arrivalLocation(prevRes);
-        const auto curDeparture = LocationUtil::departureLocation(res);
-        const auto connectedTransition = isConnectedTransition(prevRes, res);
-        qDebug() << "  current transition goes from" << LocationUtil::name(prevArrival) << "to" << LocationUtil::name(LocationUtil::arrivalLocation(res));
+        const auto curDeparture = isLocationChange ? LocationUtil::departureLocation(curRes) : LocationUtil::location(curRes);
+        const auto connectedTransition = isConnectedTransition(prevRes, curRes);
+        qDebug() << "  current transition goes from" << LocationUtil::name(prevArrival) << "to"
+                 << (isLocationChange ? LocationUtil::name(LocationUtil::arrivalLocation(curRes)) : LocationUtil::name(curDeparture)) << connectedTransition;
 
         if (!connectedSearchDone) {
-            if (!connectedTransition) {
-                qDebug() << "  aborting connectivity search, not an adjacent transition from" << LocationUtil::name(prevArrival) << "to" << LocationUtil::name(curDeparture);
-                connectedIt = m_reservations.end();
+            if (!connectedTransition && isLocationChange) {
+                qDebug() << "  aborting connectivity search, not an adjacent transition from" << LocationUtil::name(prevArrival) << "to"
+                         << LocationUtil::name(curDeparture);
                 connectedSearchDone = true;
-            } else {
+            }
+            if (connectedTransition) {
                 connectedIt = it;
             }
 
             // same location as beginIt? -> we reached the end of the trip (break)
-            const auto curArrival = LocationUtil::arrivalLocation(res);
-            if (LocationUtil::isSameLocation(beginDeparture, curArrival, LocationUtil::CityLevel)) {
-                qDebug() << "  aborting connectivity search, arrived at the start again" << LocationUtil::name(curArrival);
-                connectedSearchDone = true;
-                reachedStartAgain = true;
+            if (isLocationChange) {
+                const auto curArrival = LocationUtil::arrivalLocation(curRes);
+                if (LocationUtil::isSameLocation(beginDeparture, curArrival, LocationUtil::CityLevel)) {
+                    qDebug() << "  aborting connectivity search, arrived at the start again" << LocationUtil::name(curArrival);
+                    connectedSearchDone = true;
+                    reachedStartAgain = true;
+                }
             }
         }
 
-        if (!resNumSearchDone &&  JsonLd::canConvert<Reservation>(res)) {
-            const auto resNum = JsonLd::convert<Reservation>(res).reservationNumber();
+        if (isLocationChange && !resNumSearchDone && JsonLd::canConvert<Reservation>(curRes)) {
+            const auto resNum = JsonLd::convert<Reservation>(curRes).reservationNumber();
             if (!resNum.isEmpty()) {
-                const auto r = std::find_if(m_resNumSearch.begin(), m_resNumSearch.end(), [res, resNum](const auto &elem) {
-                    return elem.type == res.userType() && elem.resNum == resNum;
+                const auto r = std::find_if(m_resNumSearch.begin(), m_resNumSearch.end(), [curRes, resNum](const auto &elem) {
+                    return elem.type == curRes.userType() && elem.resNum == resNum;
                 });
                 if (r == m_resNumSearch.end()) {
                     // mode of transport or reservation changed: we consider this still part of the trip if connectivity
                     // search thinks this is part of the same trip too, and we are not at home again yet
                     if (connectedTransition && !LocationUtil::isSameLocation(prevArrival, beginDeparture, LocationUtil::CityLevel)) {
-                        qDebug() << "  considering transition to" << LocationUtil::name(LocationUtil::arrivalLocation(res)) << "as part of trip despite unknown reservation number";
-                        m_resNumSearch.push_back({res.userType(), resNum});
+                        qDebug() << "  considering transition to" << LocationUtil::name(LocationUtil::arrivalLocation(curRes))
+                                 << "as part of trip despite unknown reservation number";
+                        m_resNumSearch.push_back({curRes.userType(), resNum});
                         resNumIt = it;
                     } else {
                         qDebug() << "  aborting reservation number search due to mismatch";
@@ -492,9 +505,6 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
 
     // determine which search strategy found the larger result
     auto it = m_reservations.cend();
-    if (!connectedSearchDone) {
-        connectedIt = m_reservations.end();
-    }
     if (connectedIt != m_reservations.end() && resNumIt != m_reservations.end()) {
         it = std::max(connectedIt, resNumIt);
     } else {
@@ -506,6 +516,7 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
 
     if (it == m_reservations.end()) {
         qDebug() << "nothing found";
+        createAutomaticGroup({*beginIt});
         return;
     }
 
@@ -522,6 +533,10 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
             if (LocationUtil::isSameLocation(endArrival, curDeparture, LocationUtil::CityLevel)) {
                 if (beginIt != it2) {
                     qDebug() << "  removing leading appendix, starting at" << LocationUtil::name(curDeparture);
+                    QStringList appendixElems;
+                    appendixElems.reserve(std::distance(beginIt, it2));
+                    std::copy(beginIt, it2, std::back_inserter(appendixElems));
+                    createAutomaticGroup(appendixElems);
                 }
                 beginIt = it2;
                 break;
@@ -529,8 +544,9 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         }
     }
 
-    if (std::distance(beginIt, it) < MinimumTripElements - 1) {
-        qDebug() << "trip too short";
+    if (beginIt == it) {
+        qDebug() << "found empty trip";
+        createAutomaticGroup({*beginIt});
         return;
     }
 
@@ -539,26 +555,44 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
     QList<QString> elems;
     elems.reserve(std::distance(beginIt, it));
     std::copy(beginIt, it, std::back_inserter(elems));
+    createAutomaticGroup(elems);
+}
 
-    // if we are looking at an existing group, did that expand?
-    const auto groupIt = m_tripGroups.find(m_reservationToGroupMap.value(*beginIt));
+void TripGroupManager::createAutomaticGroup(const QStringList &elems)
+{
+    Q_ASSERT(!elems.empty());
+
+    QHash<QString, TripGroup>::iterator groupIt;
+    for (const auto &elem : elems) {
+        groupIt = m_tripGroups.find(m_reservationToGroupMap.value(elem));
+        if (groupIt != m_tripGroups.end()) {
+            break;
+        }
+    }
     if (groupIt != m_tripGroups.end() && groupIt.value().elements() == elems) {
         qDebug() << "existing group unchanged" << groupIt.value().name();
         return;
     }
 
-    std::set<QString> pendingGroupRemovals;
     if (groupIt == m_tripGroups.end()) {
         const auto tgId = QUuid::createUuid().toString(QUuid::WithoutBraces);
         TripGroup g;
         g.setElements(elems);
-        for (auto it2 = beginIt; it2 != it; ++it2) {
+
+        std::set<QString> pendingGroupRemovals;
+        for (const auto &elem : elems) {
             // remove overlapping/nested groups, delay this until the end though, as that will invalidate our iterators
-            const auto previousGroupId = m_reservationToGroupMap.value(*it2);
+            const auto previousGroupId = m_reservationToGroupMap.value(elem);
             if (!previousGroupId.isEmpty() && previousGroupId != tgId) {
                 pendingGroupRemovals.insert(previousGroupId);
             }
-            m_reservationToGroupMap.insert(*it2, tgId);
+        }
+        for (const auto &tgId : pendingGroupRemovals) {
+            removeTripGroup(tgId);
+        }
+
+        for (const auto &elem : elems) {
+            m_reservationToGroupMap.insert(elem, tgId);
         }
         g.setName(guessName(g.elements()));
         recomputeTripGroupTimes(g);
@@ -572,8 +606,8 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
             m_reservationToGroupMap.remove(elem);
         }
         g.setElements(elems);
-        for (auto it2 = beginIt; it2 != it; ++it2) {
-            m_reservationToGroupMap.insert(*it2, groupIt.key());
+        for (const auto &elem : elems) {
+            m_reservationToGroupMap.insert(elem, groupIt.key());
         }
         if (g.hasAutomaticName() || g.name().isEmpty()) {
             g.setName(guessName(g.elements()));
@@ -583,10 +617,6 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         g.store(fileForGroup(groupIt.key()));
         Q_EMIT tripGroupChanged(groupIt.key());
     }
-
-    for (const auto &tgId : pendingGroupRemovals) {
-        removeTripGroup(tgId);
-    }
 }
 
 void TripGroupManager::checkConsistency()
@@ -646,12 +676,42 @@ static QString destinationName(const QVariant &loc)
     return LocationUtil::name(loc);
 }
 
-QString TripGroupManager::guessDestinationFromLodging(const QStringList &elements) const
+QString TripGroupManager::guessDestinationFromEvent(const QVariantList &elements)
+{
+    // compute overall trip length
+    const auto beginDt = SortUtil::startDateTime(elements.front());
+    const auto endDt = SortUtil::endDateTime(elements.back());
+    if (!beginDt.isValid() || !endDt.isValid()) {
+        return {};
+    }
+    const auto tripLength = beginDt.secsTo(endDt);
+
+    // find an event that covers 50+% of the trip time
+    for (const auto &res : elements) {
+        if (!JsonLd::isA<EventReservation>(res)) {
+            continue;
+        }
+
+        const auto evBeginDt = SortUtil::startDateTime(res);
+        const auto evEndDt = SortUtil::endDateTime(res);
+        if (!evBeginDt.isValid() || !evEndDt.isValid()) {
+            continue;
+        }
+
+        const auto eventLength = evBeginDt.secsTo(evEndDt);
+        if (2 * eventLength > tripLength) {
+            return res.value<EventReservation>().reservationFor().value<Event>().name();
+        }
+    }
+
+    return {};
+}
+
+QString TripGroupManager::guessDestinationFromLodging(const QVariantList &elements)
 {
     // we assume that lodging indicates the actual destination, not a stopover location
     QStringList dests;
-    for (const auto &resId : elements) {
-        const auto res = m_resMgr->reservation(resId);
+    for (const auto &res : elements) {
         if (!JsonLd::isA<LodgingReservation>(res)) {
             continue;
         }
@@ -672,16 +732,16 @@ QString TripGroupManager::guessDestinationFromLodging(const QStringList &element
     return dests.join(" - "_L1);
 }
 
-bool TripGroupManager::isRoundTrip(const QStringList &elements) const
+bool TripGroupManager::isRoundTrip(const QVariantList &elements)
 {
-    const auto &depId = elements.at(0);
-    const auto &arrId = elements.constLast();
-    const auto dep = LocationUtil::departureLocation(m_resMgr->reservation(depId));
-    const auto arr = LocationUtil::arrivalLocation(m_resMgr->reservation(arrId));
+    const auto &depRes = elements.at(0);
+    const auto &arrRes = elements.constLast();
+    const auto dep = LocationUtil::departureLocation(depRes);
+    const auto arr = LocationUtil::arrivalLocation(arrRes);
     return LocationUtil::isSameLocation(dep, arr, LocationUtil::CityLevel);
 }
 
-QString TripGroupManager::guessDestinationFromTransportTimeGap(const QStringList &elements) const
+QString TripGroupManager::guessDestinationFromTransportTimeGap(const QVariantList &elements)
 {
     // we must only do this for return trips
     if (!isRoundTrip(elements)) {
@@ -693,8 +753,7 @@ QString TripGroupManager::guessDestinationFromTransportTimeGap(const QStringList
     QString destName;
     qint64 maxLength = 0;
 
-    for (const auto &resId : elements) {
-        const auto res = m_resMgr->reservation(resId);
+    for (const auto &res : elements) {
         if (!LocationUtil::isLocationChange(res)) {
             continue;
         }
@@ -716,24 +775,57 @@ QString TripGroupManager::guessDestinationFromTransportTimeGap(const QStringList
     return destName;
 }
 
+QVariant TripGroupManager::firstLocationChange(const QVariantList &elements)
+{
+    for (const auto &res : elements) {
+        if (LocationUtil::isLocationChange(res)) {
+            return res;
+        }
+    }
+    return {};
+}
+
+QVariant TripGroupManager::lastLocationChange(const QVariantList &elements)
+{
+    for (auto it = elements.rbegin(); it != elements.rend(); ++it) {
+        if (LocationUtil::isLocationChange(*it)) {
+            return *it;
+        }
+    }
+    return {};
+}
+
 QString TripGroupManager::guessName(const QStringList &elements) const
+{
+    QVariantList reservations;
+    reservations.reserve(elements.size());
+    std::transform(elements.begin(), elements.end(), std::back_inserter(reservations), [this](const auto &resId) {
+        return m_resMgr->reservation(resId);
+    });
+    return guessNameForReservations(reservations);
+}
+
+QString TripGroupManager::guessNameForReservations(const QVariantList &elements)
 {
     if (elements.isEmpty()) {
         return {};
     }
 
     // part 1: the destination of the trip
-    QString dest = guessDestinationFromLodging(elements);
+    QString dest = guessDestinationFromEvent(elements);
+    if (dest.isEmpty()) {
+        dest = guessDestinationFromLodging(elements);
+    }
     if (dest.isEmpty()) {
         dest = guessDestinationFromTransportTimeGap(elements);
     }
     if (dest.isEmpty()) {
         // two fallback cases: round-trips and one-way trips
-        const auto beginLoc = LocationUtil::departureLocation(m_resMgr->reservation(elements.at(0)));
-        const auto endLoc = LocationUtil::arrivalLocation(m_resMgr->reservation(elements.constLast()));
+        const auto beginLoc = LocationUtil::departureLocation(firstLocationChange(elements));
+        const auto endLoc = LocationUtil::arrivalLocation(lastLocationChange(elements));
         if (LocationUtil::isSameLocation(beginLoc, endLoc, LocationUtil::CityLevel)) {
             const auto middleIdx = (elements.size() - 1 + (elements.size() % 2)) / 2;
-            const auto middleRes = m_resMgr->reservation(elements.at(middleIdx));
+            const auto &middleRes = elements.at(middleIdx);
             if (LocationUtil::isLocationChange(middleRes)) {
                 dest = destinationName(LocationUtil::arrivalLocation(middleRes));
             } else {
@@ -745,11 +837,10 @@ QString TripGroupManager::guessName(const QStringList &elements) const
         }
     }
     // if none of the above worked, take anything we can find
-    for (const auto &resId : elements) {
+    for (const auto &res : elements) {
         if (!dest.isEmpty()) {
             break;
         }
-        const auto res = m_resMgr->reservation(resId);
         dest = LocationUtil::name(LocationUtil::location(res));
         if (dest.isEmpty()) {
             dest = LocationUtil::name(LocationUtil::arrivalLocation(res));
@@ -765,16 +856,24 @@ QString TripGroupManager::guessName(const QStringList &elements) const
         }
     }
 
-
     // part 2: the time range of the trip
     // three cases: within 1 month, crossing a month boundary in one year, crossing a year boundary
-    const auto beginDt = SortUtil::startDateTime(m_resMgr->reservation(elements.at(0)));
-    const auto endDt = SortUtil::endDateTime(m_resMgr->reservation(elements.constLast()));
+    const auto beginDt = SortUtil::startDateTime(elements.at(0));
+    const auto endDt = SortUtil::endDateTime(elements.constLast());
     if (beginDt.date().year() == endDt.date().year() || !endDt.isValid()) {
         if (beginDt.date().month() == endDt.date().month() || !endDt.isValid()) {
-            return i18nc("%1 is destination, %2 is the standalone month name, %3 is the year", "%1 (%2 %3)", dest, QLocale().standaloneMonthName(beginDt.date().month(), QLocale::LongFormat), beginDt.date().toString(u"yyyy"));
+            return i18nc("%1 is destination, %2 is the standalone month name, %3 is the year",
+                         "%1 (%2 %3)",
+                         dest,
+                         QLocale().standaloneMonthName(beginDt.date().month(), QLocale::LongFormat),
+                         beginDt.date().toString(u"yyyy"));
         }
-        return i18nc("%1 is destination, %2 and %3 are the standalone month names and %4 is the year", "%1 (%2/%3 %4)", dest, QLocale().monthName(beginDt.date().month(), QLocale::LongFormat), QLocale().standaloneMonthName(endDt.date().month(), QLocale::LongFormat), beginDt.date().toString(u"yyyy"));
+        return i18nc("%1 is destination, %2 and %3 are the standalone month names and %4 is the year",
+                     "%1 (%2/%3 %4)",
+                     dest,
+                     QLocale().monthName(beginDt.date().month(), QLocale::LongFormat),
+                     QLocale().standaloneMonthName(endDt.date().month(), QLocale::LongFormat),
+                     beginDt.date().toString(u"yyyy"));
     }
     return i18nc("%1 is destination, %2 and %3 are years", "%1 (%2/%3)", dest, beginDt.date().toString(u"yyyy"), endDt.date().toString(u"yyyy"));
 }
@@ -851,6 +950,7 @@ QString TripGroupManager::merge(const QString &tgId1, const QString &tgId2, cons
     m_tripGroups[tgId1] = group;
     group.store(fileForGroup(tgId1));
 
+    Q_EMIT tripGroupAboutToBeRemoved(tgId2);
     m_tripGroups.remove(tgId2);
     if (!QFile::remove(fileForGroup(tgId2))) {
         qCWarning(Log) << "Failed to delete trip group file!" << tgId2;
@@ -862,15 +962,13 @@ QString TripGroupManager::merge(const QString &tgId1, const QString &tgId2, cons
     return tgId1;
 }
 
-QString TripGroupManager::createGroup(const QStringList &elements, const QString &name)
+void TripGroupManager::removeElementsFromGroups(const QStringList &elements, const QString &excludedTgId, bool markAsExplicit)
 {
-    qCDebug(Log) << elements << name;
-
-    // find groups that contain any of the selected elements
+    // find groups that contain any of the selected elements already
     QStringList groupsToUpdate;
     for (const auto &resId : elements) {
         const auto it = m_reservationToGroupMap.constFind(resId);
-        if (it == m_reservationToGroupMap.cend()) {
+        if (it == m_reservationToGroupMap.cend() || it.value() == excludedTgId) {
             continue;
         }
         if (!groupsToUpdate.contains(it.value())) {
@@ -879,24 +977,34 @@ QString TripGroupManager::createGroup(const QStringList &elements, const QString
     }
 
     // remove elements from other groups and mark those explicitly grouped
-    for (const auto &tgId :groupsToUpdate) {
+    for (const auto &tgId : groupsToUpdate) {
         auto &tg = m_tripGroups[tgId];
         auto elems = tg.elements();
         for (const auto &resId : elements) {
             elems.removeAll(resId);
         }
-        // TODO can elems becomes empty here?
         tg.setElements(elems);
-        tg.setIsAutomaticallyGrouped(false);
+        if (elems.isEmpty()) {
+            removeTripGroup(tgId);
+            continue;
+        }
         if (tg.hasAutomaticName()) {
             tg.setName(guessName(tg.elements()));
         }
+        if (tg.isAutomaticallyGrouped() && markAsExplicit) {
+            tg.setIsAutomaticallyGrouped(false);
+        }
         recomputeTripGroupTimes(tg);
         tg.store(fileForGroup(tgId));
         Q_EMIT tripGroupChanged(tgId);
     }
+}
+
+QString TripGroupManager::createGroup(const QStringList &elements, const QString &name)
+{
+    qCDebug(Log) << elements << name;
+    removeElementsFromGroups(elements, {}, true);
 
-    // create new group
     TripGroup tg;
     tg.setElements(elements);
     tg.setIsAutomaticallyGrouped(false);
@@ -905,11 +1013,31 @@ QString TripGroupManager::createGroup(const QStringList &elements, const QString
         tg.setName(name);
         tg.setNameIsAutomatic(false);
     }
+
+    return createGroup(tg);
+}
+
+QString TripGroupManager::createEmptyGroup(const QString &name)
+{
+    qCDebug(Log) << name;
+    if (name.isEmpty()) {
+        return {};
+    }
+
+    TripGroup tg;
+    tg.setIsAutomaticallyGrouped(false);
+    tg.setName(name);
+    tg.setNameIsAutomatic(false);
+    return createGroup(tg);
+}
+
+QString TripGroupManager::createGroup(TripGroup &tg)
+{
     recomputeTripGroupTimes(tg);
 
     const auto tgId = QUuid::createUuid().toString(QUuid::WithoutBraces);
     m_tripGroups.insert(tgId, tg);
-    for (const auto &resId : elements) {
+    for (const auto &resId : tg.elements()) {
         m_reservationToGroupMap.insert(resId, tgId);
     }
 
@@ -919,4 +1047,30 @@ QString TripGroupManager::createGroup(const QStringList &elements, const QString
     return tgId;
 }
 
+void TripGroupManager::addToGroup(const QStringList &elements, const QString &tgId)
+{
+    removeElementsFromGroups(elements, tgId, false);
+
+    // add elements to tgId
+    auto &tg = m_tripGroups[tgId];
+
+    auto elems = tg.elements();
+    elems += elements;
+    elems.removeDuplicates();
+    std::sort(elems.begin(), elems.end(), [this](const auto &lhs, const auto &rhs) {
+        return SortUtil::isBefore(m_resMgr->reservation(lhs), m_resMgr->reservation(rhs));
+    });
+    tg.setElements(elems);
+
+    for (const auto &elem : elements) {
+        m_reservationToGroupMap[elem] = tgId;
+    }
+
+    tg.setIsAutomaticallyGrouped(false);
+    recomputeTripGroupTimes(tg);
+
+    tg.store(fileForGroup(tgId));
+    Q_EMIT tripGroupChanged(tgId);
+}
+
 #include "moc_tripgroupmanager.cpp"
