@@ -53,10 +53,9 @@ Q_SIGNALS:
 
 private:
     QString m_infoString;
-    Quotient::AccountRegistry m_acountRegistry;
+    Quotient::AccountRegistry m_accountRegistry;
 
     void setInfoString(const QString &infoString);
-
 };
 
-#endif //MATRIXMANAGER_H
+#endif // MATRIXMANAGER_H
